// Copyright (c)2003, TopCoder, Inc. All rights reserved
// @author BAman
// @author adic
// @version 1.0

using System;
using System.Collections;

namespace TopCoder.Util.ConfigurationManager
{
    /// <summary>
    /// Instances of implementations of this interface will be passed by the 
    /// user to specify custom handling of the errors that can occur while 
    /// loading a config file (namespace clash, property clash, critical 
    /// errors). When an error occurs the corresponding method in the callback
    /// instance is called to get the desired action to be taken from the user.
    /// 
    /// Note: clash means having the same name
    /// </summary>
    public interface ErrorCallback
    {
        /// <summary>
        /// Called in case of a namespace collision to give the
        /// user a chance to choose the action to be taken.
        /// </summary>
        /// <param name="oldNamespace">The old namespace</param>
        /// <param name="newNamespace">The new namespace</param>
        /// <returns>
        /// an ErrorAction enumeration value to specify whether
        /// the old namespace should be kept, the new namespace should
        /// replace the old one, the namespaces should be merged or
        /// an exception should be thrown
        /// </returns>
        ErrorAction NamespaceCollision(Namespace oldNamespace, 
                                       Namespace newNamespace);
        
        /// <summary>
        /// Called in case of a property collision to give the
        /// user a chance to choose the action to be taken.
        /// </summary>
        /// <param name="namespaceString">The namespace that holds both properties</param>
        /// <param name="oldProperty">The old property</param>
        /// <param name="newProperty">The new property</param>
        /// <returns>
        /// an ErrorAction enumeration value to specify whether
        /// the old property should be kept, the new property should
        /// replace the old one, the properties should be merged or
        /// an exception should be thrown
        /// </returns>
        ErrorAction PropertyCollision(Namespace namespaceString, 
                                      Property oldProperty,
                                      Property newProperty);
        
        /// <summary>
        /// Called in case of a critical error to give the
        /// user a chance to choose the action to be taken.
        /// </summary>
        /// <param name="exception">The exception that occured</param>
        /// <returns>
        /// a boolean value specifying whether the exception should be
        /// thrown or not (true - throw, false - ignore)
        /// </returns>
        bool CriticalError(Exception exception);
    }
}
