// Copyright (c)2003, TopCoder, Inc. All rights reserved
// @author BAman
// @author adic
// @version 1.0

using System;
using System.IO;

namespace TopCoder.Util.ConfigurationManager
{
    /// <summary>
    /// This interface abstract a configuration file handler.
    /// A configuration file handler has the task of identifying whether
    /// it can process the file, loading namespaces with their properties
    /// from a file and saving a set of namespaces to a file.
    /// </summary>
    internal interface FileHandler
    {
        /// <summary>
        /// Determines whether the file is supported by the handler,
        /// usually by looking at the extension, but more complex 
        /// methods can be applied if needed.
        /// </summary>
        /// 
        /// <param name="filename">The name of the file.</param>
        /// <returns>Whether the file is supported or not.</returns>
        /// 
        /// <exception cref="IOException">
        /// If an I/O error occurs.
        /// </exception>
        bool Supports(string filename);
        
        /// <summary>
        /// Loads the namespaces and properties from the file into
        /// Namespace and Property objects.
        /// </summary>
        /// 
        /// <param name="filename">The name of the file.</param>
        /// <param name="errorCallback">The error callback.</param>
        /// <returns>The populated Namespace objects</returns>
        /// 
        /// <exception cref="NamespaceClashException">
        /// If a namespace clash occurs.
        /// </exception>
        /// 
        /// <exception cref="PropertyClashException">
        /// If a property clash occurs.
        /// </exception>
        /// 
        /// <exception cref="InvalidConfigFileException">
        /// If the configuration file is invalid.
        /// </exception>
        /// 
        /// <exception cref="UnknownFormatException">
        /// If the format of the configuration file is unkown.
        /// </exception>
        /// 
        /// <exception cref="IOException">
        /// If an I/O error occurs.
        /// </exception>
        Namespace[] LoadFromFile(string filename, ErrorCallback errorCallback);
        
        /// <summary>
        /// Saves a set of Namespace objects and their properties to a file.
        /// </summary>
        /// 
        /// <param name="filename">The name of the file.</param>
        /// <param name="namespaces">The set of namespaces.</param>
        /// 
        /// <exception cref="IOException">
        /// If an I/O error occurs.
        /// </exception>
        void SaveFile(Namespace[] namespaces, string filename);
    }
}
