// Copyright (c)2005, TopCoder, Inc. All rights reserved
// @author TCSDEVELOPER
// @version 2.0

using System;
using NUnit.Framework;
using TopCoder.Util.ConfigurationManager.UnitTests;

namespace TopCoder.Util.ConfigurationManager.UnitTests
{
    /// <summary>
    /// Unit tests for the ConfigManager class v2.0.
    /// </summary>
    [TestFixture]
    public class ConfigManagerUnitTestCaseV2
    {
        /// <summary>
        /// The configuration manager instance.
        /// </summary>
        private ConfigManager cm = null;
        
        /// <summary>
        /// Set up.
        /// </summary>
        [SetUp]
        public void SetUp()
        {
            cm = ConfigManager.GetInstance();
        }

        /// <summary>
        /// Tear down.
        /// </summary>
        [TearDown]
        public void TearDown()
        {
            cm.Clear();
        }

        /// <summary>
        /// Demo for the Pluggable Handler.
        /// </summary>
        [Test]
        public void TestPluggableHandler()
        {
            Assert.IsNotNull(cm);
            ConfigFile cf = cm.LoadFile(@"..\..\test_files\database.mxml");
            Assert.IsNotNull(cf);
            Assert.AreEqual(3, cm.Namespaces.Length);
            Namespace ns = cm.GetNamespace("EmployeeDetails");
            Assert.IsNotNull(ns);
            Property prop = ns.GetProperty("FirstName");
            Assert.IsNotNull (prop);
            Assert.AreEqual ("Top", prop.Value);
            prop = ns.GetProperty("LastName");
            Assert.AreEqual ("Coder", prop.Value);
            cm.SaveFile(@"..\..\test_files\database.mxml");
        }

        /// <summary>
        /// Create instance by getting the typename from the namespace.
        /// </summary>
        [Test]
        public void CreateInstance1()
        {
            Assert.IsNotNull(cm);
            cm.LoadFile(@"..\..\test_files\testclass.xml");
            IServer server = (IServer)cm.CreateInstance(
                "TopCoder.Util.ConfigurationManager.UnitTests.TestClass",
                "Class", typeof(IServer)
                );
            Assert.IsNotNull(server);
        }

        /// <summary>
        /// Create instance by giving an argument.
        /// </summary>
        [Test]
        public void CreateInstance2()
        {
            Assert.IsNotNull(cm);
            cm.LoadFile(@"..\..\test_files\testclass.xml");
            IServer server = (IServer)cm.CreateInstance(
                "TopCoder.Util.ConfigurationManager.UnitTests.TestClass",
                "Class", new object [] {"smtp.topcoder.com", (short)25},
                typeof(IServer)
                );
            Assert.IsNotNull(server);
        }

        /// <summary>
        /// Test the CreateInstance() method for creating a class without constructor 
        /// arguments. For v2.0.
        /// </summary>
        [Test]
        public void CreateInstance3()
        {
            Assert.IsNotNull(cm);
            cm.LoadFile(@"..\..\test_files\testclass.xml");
            Assert.AreEqual (new SmtpServer().ToString(), (cm.CreateInstance(
                "TopCoder.Util.ConfigurationManager.UnitTests.TestClass",
                "Class", "Assembly",
                typeof(object)).ToString())
                );
        }

        /// <summary>
        /// CreateInstance() on a class that has a constructor which accepts arguments. For v2.0.
        /// </summary>
        [Test]
        public void CreateInstance4()
        {
            Assert.IsNotNull(cm);
            cm.LoadFile(@"..\..\test_files\testclass.xml");
            Assert.AreEqual (new SmtpServer("smtp.topcoder.com", 25).ToString(), (cm.CreateInstance(
                "TopCoder.Util.ConfigurationManager.UnitTests.TestClass",
                "Class", "Assembly", new object [] {"smtp.topcoder.com", (short)25},
                typeof(object)).ToString())
                );
        }



        /// <summary>
        /// Test the GetValue()  suite of methods. For v2.0.
        /// </summary>
        [Test]
        public void TestNewGetValue()
        {
            Assert.IsNotNull(cm);
            ConfigFile cf = cm.LoadFile(@"..\..\test_files\test4.xml");
            Assert.IsNotNull(cf);
            string ns = "TopCoder.Util.ConfigurationManager.FunctionalTests4";
            Assert.AreEqual("v3", cm.GetValue(ns, "p3", typeof(string)), "ConfigurationManager.GetValue(ns, property, type) failed.");
            Assert.AreEqual("3", cm.GetValue(ns, "p4", typeof(string)), "ConfigurationManager.GetValue(ns, property, type) failed.");
            Assert.AreEqual(3, cm.GetValue(ns, "p4", typeof(int)), "ConfigurationManager.GetValue(ns, property, type) failed.");
            Assert.AreEqual(3, cm.GetIntValue(ns, "p4"), "ConfigurationManager.GetIntValue(ns, property, type) failed.");
            Assert.AreEqual(3.0, cm.GetDoubleValue(ns, "p4"), "ConfigurationManager.GetDoubleValue(ns, property, type) failed.");
        }

        /// <summary>
        /// Test the GetValues()  suite of methods. For v2.0.
        /// </summary>
        [Test]
        public void TestNewGetValues()
        {
            Assert.IsNotNull(cm);
            ConfigFile cf = cm.LoadFile(@"..\..\test_files\test4.xml");
            Assert.IsNotNull(cf);
            string ns = "TopCoder.Util.ConfigurationManager.FunctionalTests4";
            AreEqual(new string [] {"v1", "v2"}, cm.GetValues(ns, "p1", typeof(string)), "ConfigurationManager.GetValue(ns, property, type) failed.");
            AreEqual(new string [] {"1", "2"}, cm.GetValues(ns, "p2", typeof(string)), "ConfigurationManager.GetValue(ns, property, type) failed.");
            AreEqual(new int[] {1, 2}, cm.GetValues(ns, "p2", typeof(int)), "ConfigurationManager.GetValue(ns, property, type) failed.");
            AreEqual(new int[] {1, 2}, cm.GetIntValues(ns, "p2"), "ConfigurationManager.GetIntValues(ns, property, type) failed.");
            AreEqual(new double[] {1.0, 2.0}, cm.GetDoubleValues(ns, "p2"), "ConfigurationManager.GetDoubleValues(ns, property, type) failed.");
        }

        /// <summary>
        /// Verifies two arrays are equivalent.
        /// </summary>
        /// <param name="a">the first array</param>
        /// <param name="b">the second array</param>
        /// <param name="message">the error message</param>
        private void AreEqual(Array a, Array b, string message)
        {
            Assert.AreEqual(a.Length, b.Length, message);
            for (int i = 0; i < a.Length; ++i)
            {
                Assert.AreEqual(a.GetValue(i), b.GetValue(i), message);
            }
        }

        enum Color 
        {
            RED,
            BLUE,
            GREEN
        }

        /// <summary>
        /// Test the DefaultStringConverter object with GetValue() method to get 
        /// an enum value. For v2.0.
        /// </summary>
        [Test]
        public void TestGetEnumValue()
        {
            Assert.IsNotNull(cm);
            ConfigFile cf = cm.LoadFile(@"..\..\test_files\test4.xml");
            Assert.IsNotNull(cf);
            string ns = "TopCoder.Util.ConfigurationManager.FunctionalTests4";
            Color myColor = (Color)cm.GetValue(ns, "mycolor", typeof(Color));
            Assert.IsTrue(myColor == Color.GREEN);
        }

        /// <summary>
        /// Test the GetValue() method with a user defined StringConverter. For v2.0.
        /// </summary>
        [Test]
        public void TestStringConverter()
        {
            Assert.IsNotNull(cm);
            ConfigFile cf = cm.LoadFile(@"..\..\test_files\test4.xml");
            Assert.IsNotNull(cf);
            string ns = "TopCoder.Util.ConfigurationManager.FunctionalTests4";
            IStringConverter myStringConverter = new StringConverter1();
            Color myColor = (Color)cm.GetValue(ns, "mycolor", typeof(Color), myStringConverter);
            Assert.IsTrue(myColor == Color.GREEN);
        }

        /// <summary>
        /// Test the GetValue() method with a user defined ParsingPreference object. For v2.0.
        /// </summary>
        [Test]
        public void TestParsingPreference()
        {
            Assert.IsNotNull(cm);
            ConfigFile cf = cm.LoadFile(@"..\..\test_files\test4.xml");
            Assert.IsNotNull(cf);
            string ns = "TopCoder.Util.ConfigurationManager.FunctionalTests4";
            IParsingPreference defaultPP = new ParsingPreference();
            string val = cm.GetValue(ns, "mycolor", defaultPP);
            Assert.IsTrue ("2" == val);

            IParsingPreference myPP1 = new MyParsingPreference1();
            val = cm.GetValue(ns, "mycolor", myPP1);
            Assert.IsTrue ("GREEN" == val);
        }
    }
}
