// Copyright (c)2003, TopCoder, Inc. All rights reserved
// @author BAman
// @author adic
// @version 1.0

using System;
using System.IO;
using NUnit.Framework;

namespace TopCoder.Util.ConfigurationManager.UnitTests
{
    /// <summary>
    /// Test implementation of the ErrorCallback interface with replace old
    /// </summary>
    public class TestErrorCallback1 : MarshalByRefObject, ErrorCallback 
    {
        /// <summary>
        /// Called in case of a namespace collision to give the
        /// user a chance to choose the action to be taken.
        /// Returns always ErrorAction.ThrowException.
        /// </summary>
        /// <param name="oldNamespace">The old namespace</param>
        /// <param name="newNamespace">The new namespace</param>
        /// <returns>
        /// an ErrorAction enumeration value to specify whether
        /// the old namespace should be kept, the new namespace should
        /// replace the old one, the namespaces should be merged or
        /// an exception should be thrown
        /// </returns>
        public ErrorAction NamespaceCollision(Namespace oldNamespace, 
            Namespace newNamespace)
        {
            return ErrorAction.ReplaceOld;  
        }
        
        /// <summary>
        /// Called in case of a property collision to give the
        /// user a chance to choose the action to be taken.
        /// Returns always ErrorAction.ThrowException.
        /// </summary>
        /// <param name="namespaceString">The namespace that holds both properties</param>
        /// <param name="oldProperty">The old property</param>
        /// <param name="newProperty">The new property</param>
        /// <returns>
        /// an ErrorAction enumeration value to specify whether
        /// the old property should be kept, the new property should
        /// replace the old one, the properties should be merged or
        /// an exception should be thrown
        /// </returns>
        public ErrorAction PropertyCollision(Namespace namespaceString, 
            Property oldProperty,
            Property newProperty)
        {
            return ErrorAction.ReplaceOld;
        }
        
        /// <summary>
        /// Called in case of a critical error to give the
        /// user a chance to choose the action to be taken.
        /// Returns always true.
        /// </summary>
        /// <param name="exception">The exception that occured</param>
        /// <returns>
        /// a boolean value specifying whether the exception should be
        /// thrown or not (true - throw, false - ignore)
        /// </returns>
        public bool CriticalError(Exception exception)
        {
            return true;
        }
    }

    /// <summary>
    /// Test implementation of the ErrorCallback interface with keep old
    /// </summary>
    public class TestErrorCallback2 : MarshalByRefObject, ErrorCallback 
    {
        /// <summary>
        /// Called in case of a namespace collision to give the
        /// user a chance to choose the action to be taken.
        /// Returns always ErrorAction.ThrowException.
        /// </summary>
        /// <param name="oldNamespace">The old namespace</param>
        /// <param name="newNamespace">The new namespace</param>
        /// <returns>
        /// an ErrorAction enumeration value to specify whether
        /// the old namespace should be kept, the new namespace should
        /// replace the old one, the namespaces should be merged or
        /// an exception should be thrown
        /// </returns>
        public ErrorAction NamespaceCollision(Namespace oldNamespace, 
            Namespace newNamespace)
        {
            return ErrorAction.KeepOld;  
        }
        
        /// <summary>
        /// Called in case of a property collision to give the
        /// user a chance to choose the action to be taken.
        /// Returns always ErrorAction.ThrowException.
        /// </summary>
        /// <param name="namespaceString">The namespace that holds both properties</param>
        /// <param name="oldProperty">The old property</param>
        /// <param name="newProperty">The new property</param>
        /// <returns>
        /// an ErrorAction enumeration value to specify whether
        /// the old property should be kept, the new property should
        /// replace the old one, the properties should be merged or
        /// an exception should be thrown
        /// </returns>
        public ErrorAction PropertyCollision(Namespace namespaceString, 
            Property oldProperty,
            Property newProperty)
        {
            return ErrorAction.KeepOld;
        }
        
        /// <summary>
        /// Called in case of a critical error to give the
        /// user a chance to choose the action to be taken.
        /// Returns always true.
        /// </summary>
        /// <param name="exception">The exception that occured</param>
        /// <returns>
        /// a boolean value specifying whether the exception should be
        /// thrown or not (true - throw, false - ignore)
        /// </returns>
        public bool CriticalError(Exception exception)
        {
            return true;
        }
    }

    /// <summary>
    /// Test implementation of the ErrorCallback interface with merge
    /// </summary>
    public class TestErrorCallback3 : MarshalByRefObject, ErrorCallback 
    {
        /// <summary>
        /// Called in case of a namespace collision to give the
        /// user a chance to choose the action to be taken.
        /// Returns always ErrorAction.ThrowException.
        /// </summary>
        /// <param name="oldNamespace">The old namespace</param>
        /// <param name="newNamespace">The new namespace</param>
        /// <returns>
        /// an ErrorAction enumeration value to specify whether
        /// the old namespace should be kept, the new namespace should
        /// replace the old one, the namespaces should be merged or
        /// an exception should be thrown
        /// </returns>
        public ErrorAction NamespaceCollision(Namespace oldNamespace, 
            Namespace newNamespace)
        {
            return ErrorAction.Merge;  
        }
        
        /// <summary>
        /// Called in case of a property collision to give the
        /// user a chance to choose the action to be taken.
        /// Returns always ErrorAction.ThrowException.
        /// </summary>
        /// <param name="namespaceString">The namespace that holds both properties</param>
        /// <param name="oldProperty">The old property</param>
        /// <param name="newProperty">The new property</param>
        /// <returns>
        /// an ErrorAction enumeration value to specify whether
        /// the old property should be kept, the new property should
        /// replace the old one, the properties should be merged or
        /// an exception should be thrown
        /// </returns>
        public ErrorAction PropertyCollision(Namespace namespaceString, 
            Property oldProperty,
            Property newProperty)
        {
            return ErrorAction.Merge;
        }
        
        /// <summary>
        /// Called in case of a critical error to give the
        /// user a chance to choose the action to be taken.
        /// Returns always true.
        /// </summary>
        /// <param name="exception">The exception that occured</param>
        /// <returns>
        /// a boolean value specifying whether the exception should be
        /// thrown or not (true - throw, false - ignore)
        /// </returns>
        public bool CriticalError(Exception exception)
        {
            return true;
        }
    }

    /// <summary>
    /// Unit tests for the ErrorCallback interface
    /// </summary>
    [TestFixture]
    public class ErrorCallbackUnitTestCase
    {
        /// <summary>
        /// The configuration manager instance.
        /// </summary>
        private ConfigManager cm = null;
        
        /// <summary>
        /// Set up.
        /// </summary>
        [SetUp]
        public void SetUp()
        {
            cm = ConfigManager.GetInstance();
        }

        /// <summary>
        /// Tear down.
        /// </summary>
        [TearDown]
        public void TearDown()
        {
            // Just in case these actually pop into existence for some reason
            File.Delete(@"..\..\test_files\alsobroken.ini");
            File.Delete(@"..\..\test_files\evenmorebroken.ini");
			cm.Clear();
        }

        /// <summary>
        /// Tests the ErrorCallback's NamespaceCollision method with KeepOld ErrorAction
        /// </summary>
        [Test]
        public void TestNamespaceCollisionWithKeepOld()
        {
            Assertion.AssertNotNull(cm);
            ConfigFile cf = cm.LoadFile(@"..\..\test_files\broken.ini", new TestErrorCallback1());
            
            Namespace namespaceObject = cm.GetNamespace("section1");
            Assertion.AssertNotNull(namespaceObject);
            Property prop = namespaceObject.GetProperty("prop5");
            Assertion.AssertNotNull(prop);
            Assertion.AssertEquals("prop5", prop.Name);

            cm.RemoveFile(@"..\..\test_files\broken.ini");
        }

        /// <summary>
        /// Tests the ErrorCallback's NamespaceCollision method with ReplaceOld ErrorAction
        /// </summary>
        [Test]
        public void TestNamespaceCollisionWithReplaceOld()
        {
            Assertion.AssertNotNull(cm);
            ConfigFile cf = cm.LoadFile(@"..\..\test_files\broken.ini", new TestErrorCallback2());
            
            Namespace namespaceObject = cm.GetNamespace("section1");
            Assertion.AssertNotNull(namespaceObject);
            Property prop = namespaceObject.GetProperty("prop3");
            Assertion.AssertNotNull(prop);
            Assertion.AssertEquals("prop3", prop.Name);

            cm.RemoveFile(@"..\..\test_files\broken.ini");
        }

        /// <summary>
        /// Tests the ErrorCallback's NamespaceCollision method with Merge ErrorAction
        /// </summary>
        [Test]
        public void TestNamespaceCollisionWithMerge()
        {
            Assertion.AssertNotNull(cm);
            ConfigFile cf = cm.LoadFile(@"..\..\test_files\broken.ini", new TestErrorCallback3());
            
            Namespace namespaceObject = cm.GetNamespace("section1");
            Assertion.AssertNotNull(namespaceObject);
            Property prop = namespaceObject.GetProperty("prop3");
            Assertion.AssertNotNull(prop);
            Assertion.AssertEquals("prop3", prop.Name);

            namespaceObject = cm.GetNamespace("section1");
            Assertion.AssertNotNull(namespaceObject);
            prop = namespaceObject.GetProperty("prop5");
            Assertion.AssertNotNull(prop);
            Assertion.AssertEquals("prop5", prop.Name);

            try
            {
                cm.SaveFileTo(@"..\..\test_files\broken.ini",@"..\..\test_files\alsobroken.ini");
				cm.RemoveFile(@"..\..\test_files\broken.ini");
				// The exception should get thrown before we reach this line
                Assertion.Fail();
            }
            catch (MergeException)
            {
                // Good, we wanted this exception, ignore it
            }

			// The file is still loaded at this point because the merge is preventing it from
			// being unloaded, however the TearDown will call cm.Clear(), so the state will
			// still be preserved
		}

        /// <summary>
        /// Tests the ErrorCallback's NamespaceCollision method with ThrowException ErrorAction
        /// </summary>
        [Test]
        public void TestNamespaceCollisionWithThrowException()
        {
            Assertion.AssertNotNull(cm);
            try
            {
                cm.LoadFile(@"..\..\test_files\broken.ini", new DefaultErrorCallback());
                // The exception should get thrown before we reach this line
                Assertion.Fail();
            }
            catch (NamespaceClashException)
            {
                // Good, we wanted this exception, ignore it
            }
            finally
            {
                cm.RemoveFile(@"..\..\test_files\broken.ini");
            }
        }

        /// <summary>
        /// Tests the ErrorCallback's PropertyCollision method with KeepOld ErrorAction
        /// </summary>
        [Test]
        public void TestPropertyCollisionWithKeepOld()
        {
            Assertion.AssertNotNull(cm);
            ConfigFile cf = cm.LoadFile(@"..\..\test_files\morebroken.ini", new TestErrorCallback1());
            
            Namespace namespaceObject = cm.GetNamespace("section1");
            Assertion.AssertNotNull(namespaceObject);
            Property prop = namespaceObject.GetProperty("prop1");
            Assertion.AssertNotNull(prop);
            Assertion.AssertEquals("prop1", prop.Name);
            Assertion.AssertEquals(2, prop.Values.Length);

            namespaceObject = cm.GetNamespace("section2");
            Assertion.AssertNotNull(namespaceObject);
            prop = namespaceObject.GetProperty("prop5");
            Assertion.AssertNotNull(prop);
            Assertion.AssertEquals("prop5", prop.Name);
            Assertion.AssertEquals(3, prop.Values.Length);

            cm.RemoveFile(@"..\..\test_files\morebroken.ini");

        }

        /// <summary>
        /// Tests the ErrorCallback's PropertyCollision method with ReplaceOld ErrorAction
        /// </summary>
        [Test]
        public void TestPropertyCollisionWithReplaceOld()
        {
            Assertion.AssertNotNull(cm);
            ConfigFile cf = cm.LoadFile(@"..\..\test_files\morebroken.ini", new TestErrorCallback2());
            
            Namespace namespaceObject = cm.GetNamespace("section1");
            Assertion.AssertNotNull(namespaceObject);
            Property prop = namespaceObject.GetProperty("prop1");
            Assertion.AssertNotNull(prop);
            Assertion.AssertEquals("prop1", prop.Name);
            Assertion.AssertEquals(1, prop.Values.Length);

            namespaceObject = cm.GetNamespace("section2");
            Assertion.AssertNotNull(namespaceObject);
            prop = namespaceObject.GetProperty("prop5");
            Assertion.AssertNotNull(prop);
            Assertion.AssertEquals("prop5", prop.Name);
            Assertion.AssertEquals(2, prop.Values.Length);

            cm.RemoveFile(@"..\..\test_files\morebroken.ini");
        }

        /// <summary>
        /// Tests the ErrorCallback's PropertyCollision method with Merge ErrorAction
        /// </summary>
        [Test]
        public void TestPropertyCollisionWithMerge()
        {
            Assertion.AssertNotNull(cm);
            ConfigFile cf = cm.LoadFile(@"..\..\test_files\morebroken.ini", new TestErrorCallback3());
            
            Namespace namespaceObject = cm.GetNamespace("section1");
            Assertion.AssertNotNull(namespaceObject);
            Property prop = namespaceObject.GetProperty("prop1");
            Assertion.AssertNotNull(prop);
            Assertion.AssertEquals("prop1", prop.Name);
            Assertion.AssertEquals(3, prop.Values.Length);

            namespaceObject = cm.GetNamespace("section2");
            Assertion.AssertNotNull(namespaceObject);
            prop = namespaceObject.GetProperty("prop5");
            Assertion.AssertNotNull(prop);
            Assertion.AssertEquals("prop5", prop.Name);
            Assertion.AssertEquals(5, prop.Values.Length);
        
            try
            {
                cm.SaveFileTo(@"..\..\test_files\morebroken.ini",@"..\..\test_files\evenmorebroken.ini");
				cm.RemoveFile(@"..\..\test_files\morebroken.ini");
				// The exception should get thrown before we reach this line
                Assertion.Fail();
            }
            catch (MergeException)
            {
                // Good, we wanted this exception, ignore it
            }

			// The file is still loaded at this point because the merge is preventing it from
			// being unloaded, however the TearDown will call cm.Clear(), so the state will
			// still be preserved
        }

        /// <summary>
        /// Tests the ErrorCallback's PropertyCollision method with ThrowException ErrorAction
        /// </summary>
        [Test]
        public void TestPropertyCollisionWithThrowException()
        {
            Assertion.AssertNotNull(cm);
            try
            {
                cm.LoadFile(@"..\..\test_files\morebroken.ini", new DefaultErrorCallback());
                // The exception should get thrown before we reach this line
                Assertion.Fail();
            }
            catch (PropertyClashException)
            {
                // Good, we wanted this exception, ignore it
            }
            finally
            {
                cm.RemoveFile(@"..\..\test_files\morebroken.ini");
            }
        }
    }
}
