// Copyright (c)2005, TopCoder, Inc. All rights reserved
// @author TCSDEVELOPER
// @version 2.0

using System;
using System.Collections;
using TopCoder.Util.ConfigurationManager;
using System.Data.OleDb;

// This file should typically be part of a client application that uses
// ConfigurationManager. This class uses the Namespace & Property classes
// declared in ConfigurationManager to load the namespace and property-values
// information from an external datasource (like a database, xl sheet, mdb file or 
// an xml file). However, the constructors of Namespace and Property classes are 
// declared as internal and so a client application cannot implement the IDataHandler 
// in an external assembly.
namespace TopCoder.Util.ConfigurationManager
{
    /// <summary>
    /// SamplePluggableDataHandler class demonstrates how to implement
    /// the IDataHandler interface. In this sample, the data is maintained in 
    /// an MDB file.
    /// </summary>
    public class SamplePluggableDataHandler : IDataHandler
    {
        /// <summary>
        /// Holds the name of the namespace table in the database.
        /// </summary>
        private string namespaceTable = null;

        /// <summary>
        /// The name of the table that holds the property names.
        /// </summary>
        private string propertiesTable = null;

        /// <summary>
        /// The values table that holds all the values.
        /// </summary>
        private string valuesTable = null;

        /// <summary>
        /// The connection object.
        /// </summary>
        private OleDbConnection conn = null;

        /// <summary>
        /// Constructor which gets called with a set of name-value pairs that describe
        /// the data source.
        /// </summary>
        /// <param name="dsDetails">Name-Value pairs that describe the datasource.</param>
        public SamplePluggableDataHandler (IDictionary dsDetails)
        {
            string connectionString = (string)dsDetails["connectionUrl"];
            namespaceTable = (string)dsDetails["namespaceTable"];
            propertiesTable = (string)dsDetails["propertyTable"];
            valuesTable = (string)dsDetails["valueTable"];
            try
            {
                conn = new OleDbConnection(connectionString);
            }
            catch (OleDbException e)
            {
                throw new Exception (e.Message, e);
            }
        }

        /// <summary>
        /// IDataHandler.Load() implementation.
        /// </summary>
        /// <param name="errorCallback">The object that handles errors.</param>
        /// <returns>A list of namespaces loaded from the datasource.</returns>
        public IList Load (ErrorCallback errorCallback)
        {
            IDictionary namespaces = new Hashtable();
            OleDbCommand cmd = new OleDbCommand();

            conn.Open();
            cmd.Connection = conn;
            cmd.CommandText = "SELECT * FROM " + namespaceTable;
            OleDbDataReader reader = cmd.ExecuteReader();
            while (reader.Read())
            {
                object ns_id = reader.GetValue(0);
                string ns_name = (string)reader.GetValue(1);
                Namespace ns = new Namespace(ns_name);
                namespaces.Add(ns_id, ns);
            }

            reader.Close();

            foreach (object key in namespaces.Keys)
            {
                string query =  "SELECT * FROM " + propertiesTable + 
                    " WHERE namespace_id = " + key;
                Namespace ns = (Namespace)namespaces[key];
                cmd.CommandText = query;
                reader = cmd.ExecuteReader();
                IDictionary props = new Hashtable();

                while (reader.Read())
                {
                    object prop_id = reader.GetValue(0);
                    string prop_name = (string)reader.GetValue(2);
                    props.Add(prop_id, prop_name);
                }

                reader.Close();

                foreach (object prop_id in props.Keys)
                {
                    ArrayList values = new ArrayList();

                    query = "SELECT * FROM " + valuesTable +
                        " WHERE prop_id = " + prop_id;
                    cmd.CommandText = query;
                    reader = cmd.ExecuteReader();
                    while (reader.Read())
                    {
                        object data = reader.GetValue(2);
                        string val = "";
                        if (data != System.DBNull.Value)
                        {
                            val = (string)data;
                        }
                        values.Add(val);
                    }
                    reader.Close();
                    string prop_name = (string)props[prop_id];
                    Property prop = new Property(prop_name, (string[])values.ToArray(typeof(string)));
                    ns.AddProperty(prop, errorCallback);
                }
            }

            return new ArrayList(namespaces.Values);;
        }

        /// <summary>
        /// Saves the namespaces and their associated properties and values 
        /// to the datasource.
        /// </summary>
        /// <param name="namespaces">List of namespaces to be saved.</param>
        public void Save (IList namespaces)
        {
            if (namespaces != null)
            {
                object ns_id;

                conn.Open();

                foreach (Namespace ns in namespaces)
                {
                    ns_id = AddNamespace (ns);
                    Property [] props = ns.Properties;
                    foreach (Property prop in props)
                    {
                        object prop_id = AddProperty (ns_id, prop);
                        string [] vals = prop.Values;
                        foreach (string str in vals)
                        {
                            AddValue (prop_id, str);  
                        }
                    }
                }

            }
        }

        /// <summary>
        /// If the namespace doesn't exist in the database, it is added.
        /// </summary>
        /// <param name="ns">Namespace to be added.</param>
        /// <returns>The primary key of the namespace.</returns>
        object AddNamespace (Namespace ns)
        {
            string query;
            object ns_id;
            OleDbCommand cmd = new OleDbCommand();


            cmd.Connection = conn;
            query = "SELECT id FROM " + namespaceTable + " WHERE name = '" + ns.Name + "'";
            cmd.CommandText = query;
            OleDbDataReader reader = cmd.ExecuteReader();
            reader.Read();
            ns_id = reader.GetValue(0);
            reader.Close();
            if (ns_id == System.DBNull.Value)
            {
                query = "INSERT INTO " + namespaceTable + " (name) " + 
                    " VALUES('" + ns.Name + "')";
                cmd.CommandText = query;
                cmd.ExecuteNonQuery();
                query = "SELECT id FROM " + namespaceTable + " WHERE name = '" + ns.Name + "'";
                cmd.CommandText = query;            
                reader = cmd.ExecuteReader();
                reader.Read();
                ns_id = reader.GetValue(0);
                reader.Close();
            }

            return ns_id;
        }

        /// <summary>
        /// Adds a property for a particular namespace, if it doesn't exist.
        /// </summary>
        /// <param name="namespace_id">Namespace to which the property is to be added.</param>
        /// <param name="prop">Property to be added</param>
        /// <returns>The primary key of the added property.</returns>
        object AddProperty (object namespace_id, Property prop)
        {
            string query;
            object prop_id;
            OleDbCommand cmd = new OleDbCommand();

            cmd.Connection = conn;
            query = "SELECT id FROM " + propertiesTable + 
                    " WHERE namespace_id = " + namespace_id +
                    " AND name = '" + prop.Name + "'";
            cmd.CommandText = query;
            OleDbDataReader reader = cmd.ExecuteReader();
            reader.Read();
            prop_id = reader.GetValue(0);
            reader.Close();
            if (prop_id == System.DBNull.Value)
            {
                query = "INSERT INTO " + propertiesTable + " (namespace_id, name) " +
                    " VALUES(" + namespace_id + ", '" + prop.Name + "')";
                cmd.CommandText = query;
                cmd.ExecuteNonQuery();
                query = "SELECT id FROM " + propertiesTable + " WHERE name = '" + prop.Name + "'";
                cmd.CommandText = query;
                reader = cmd.ExecuteReader();
                reader.Read();
                prop_id= reader.GetValue(0);
                reader.Close();
            }

            return prop_id;
        }

        /// <summary>
        /// Adds a value (if it doesn't exist) for a property.
        /// </summary>
        /// <param name="prop_id">Property id to which the value is to be added.</param>
        /// <param name="val">The value to be added.</param>
        void AddValue (object prop_id, string val)
        {
            string query;
            object val_id;
            OleDbCommand cmd = new OleDbCommand();

            cmd.Connection = conn;
            query = "SELECT id FROM " + valuesTable + 
                " WHERE prop_id = " + prop_id +
                " AND value = '" + val + "'";
            cmd.CommandText = query;
            OleDbDataReader reader = cmd.ExecuteReader();
            reader.Read();
            val_id = reader.GetValue(0);
            reader.Close();
            if (val_id == System.DBNull.Value)
            {
                query = "INSERT INTO " + valuesTable + " (prop_id, value) " +
                    " VALUES(" + prop_id + ", '" + val + "')";
                cmd.CommandText = query;
                cmd.ExecuteNonQuery();
            }
        }
            

    }
   
}