using System;
using System.Xml;
using System.IO;
using System.Diagnostics;
using System.Threading;

using Microsoft.Win32;

using NAnt.Core;
using NAnt.Core.Attributes;
using NAnt.Core.Types;

namespace TopCoder.Util.SandCastle
{
    /// <summary>
    /// This class is custom task for nant. It allow to generate documentation for .net assemblies from NANT
    /// </summary>
    [TaskName("sandcastle")]
    public class SandCastleDocumenter : Task
    {
        // Private fields.
        string assembly = null;
        string xmldoc = null;
        string outFolder = "docs";
        string tempFolder = null;
        string sandcastle = null;
        string hhc = null;
        string sysPath = null;
        bool is11 = false;

        /// <summary>
        /// Get or change the assembly that will be documented.
        /// </summary>
        [TaskAttribute("Assembly", Required=true)]
        public string Assembly
        {
            get { return assembly; }
            set { assembly = value; }
        }

        /// <summary>
        /// Get or change the folder where this tool will write its output.
        /// </summary>
        [TaskAttribute("OutputFolder", Required=false)]
        public string OutFolder
        {
            get { return outFolder; }
            set { outFolder = value; }
        }

        /// <summary>
        /// Get or change the folder that this tool will utilize during documentation's construction.
        /// </summary>
        [TaskAttribute("TempFolder", Required=false)]
        public string TempFolder
        {
            get { return tempFolder; }
            set { tempFolder = value; }
        }

        /// <summary>
        /// Get or change the folder where SandCastle is installed.
        /// </summary>
        [TaskAttribute("SandcastleRoot", Required=false)]
        public string Sandcastle
        {
            get { return sandcastle; }
            set { sandcastle = value; }
        }

        /// <summary>
        /// Get or change the folder where hhc.exe (part of "HTML Help Workshop") is installed.
        /// </summary>
        [TaskAttribute("HhcRoot", Required=false)]
        public string Hhc
        {
            get { return hhc; }
            set { hhc = value; }
        }

        /// <summary>
        /// Get or change a flag that indicates if input assembly was compiled with .net 1.1 compilers.
        /// </summary>
        [TaskAttribute("IsFor11Framework", Required=false)]
        public bool Is11
        {
            get { return is11; }
            set { is11 = value; }
        }

        /// <summary>
        /// Helper method to initilize this class from outside nant (used by console tool)
        /// </summary>
        public void InitializeDocumenter()
        {
            InitializeTask(null);
        }

        /// <summary>
        /// Helper method that assure that all parameters received are valid and that all mandatory parameters
        /// are there
        /// </summary>
        /// <param name="taskNode">Not used</param>
        protected override void InitializeTask(XmlNode taskNode)
        {
            // Check that source exists.
            if (assembly == null || !File.Exists(assembly))
                throw new BuildException("The source assembly does not exist or was not defined", Location);
            assembly = Path.GetFullPath(assembly);

            // Check that XML documentation exists
            xmldoc = Path.ChangeExtension(assembly, ".XML");
            if (!File.Exists(xmldoc))
                throw new BuildException("Cannot find XML Documentation for indicated assembly", Location);

            // Check (or create) the folder that will hold the output.
            if (outFolder == null || outFolder.Length == 0)
                outFolder = "docs";
            if (!Directory.Exists(outFolder))
            {
                try { Directory.CreateDirectory(outFolder); }
                catch (Exception e)
                {
                    throw new BuildException("Cannot create output folder", Location, e);
                }
            }
            outFolder = Path.GetFullPath(outFolder);

            // Determine and check the temporary folder to use.
            if (tempFolder != null && !Directory.Exists(tempFolder))
                tempFolder = null;
            if (tempFolder == null)
                tempFolder = Environment.GetEnvironmentVariable("TEMP");
            if (tempFolder == null || tempFolder.Length == 0)
                tempFolder = Environment.GetEnvironmentVariable("TMP");
            if (tempFolder == null || !Directory.Exists(tempFolder))
                throw new BuildException("The temporary folder does not exists (possibly, check TEMP environment var", Location);
            tempFolder += "\\scastle";
            tempFolder = Path.GetFullPath(tempFolder);

            // Identify sandcastle's installation.
            string progFolder = (string)
                Registry.LocalMachine.OpenSubKey(@"software\microsoft\windows\currentversion").GetValue("programfilesdir");
            if (sandcastle == null)
                sandcastle = progFolder + @"\sandcastle\";
            if(!File.Exists(sandcastle + "ProductionTools\\MRefBuilder.exe"))
                throw new BuildException("'SandCastle' is not found where indicated, or in its standard location", Location);

            // Identify HHC.EXE installation.
            if (hhc == null)
                hhc = progFolder + @"\\HTML Help Workshop\";
            if(!File.Exists(hhc + "hhc.exe"))
                throw new BuildException("'HTML Help Workshop' is not found where indicated, or in its standard location", Location);

            // Add sandcastle to the path, else, some dlls are not loaded
            sysPath = Environment.GetEnvironmentVariable("PATH");
            if (!sysPath.ToLower().Contains("\\sandcastle"))
            {
                sysPath += ";" + sandcastle + "ProductionsTools";
                Environment.SetEnvironmentVariable("PATH", sysPath);
            }
            if (Environment.GetEnvironmentVariable("DXROOT") == null)
                Environment.SetEnvironmentVariable("DXROOT", sandcastle);
        }

        /// <summary>
        /// Method that execute a command as a sub process.
        /// </summary>
        /// <param name="cmdLine">The command to execute</param>
        /// <param name="args">Arguments form the command</param>
        private void ExecuteCommand(string cmdLine, string args)
        {
            ProcessStartInfo pInfo = new ProcessStartInfo(cmdLine, args);
            pInfo.CreateNoWindow = true;
            pInfo.ErrorDialog = false;

            /*
            pInfo.UseShellExecute = false;
            pInfo.EnvironmentVariables["PATH"] = sysPath;
            pInfo.EnvironmentVariables["DXROOT"] = sandcastle;
            pInfo.RedirectStandardOutput = true;
            pInfo.RedirectStandardError = true;
            */

            Process proc = new Process();
            proc.StartInfo = pInfo;
            proc.Start();
            proc.WaitForExit();
            
            // Report any error.
            if(proc.ExitCode != 0)
            {
                string msg =
                    "Error " + proc.ExitCode +
                    " executing '" + cmdLine +
                    "' with following arguments: '" + args;
                /*
                msg += "Std Output:\n" + proc.StandardOutput.ReadToEnd() + "\n" +
                    "Std error:\n" + proc.StandardError.ReadToEnd() + "\n";
                */
                throw new BuildException(msg);
            }
        }

        /// <summary>
        /// Helper method to recursively copy all the content of a folder onto another one.
        /// </summary>
        /// <param name="srcPath">The folder from which to copy files</param>
        /// <param name="dstPath">The folder onto which to copy files</param>
        void CopyFiles(string srcPath, string dstPath)
        {
            string[] files = Directory.GetFiles(srcPath, "*.*", SearchOption.TopDirectoryOnly);
            foreach (string file in files)
            {
                string destin = dstPath + "\\" + Path.GetFileName(file);
                File.Copy(file, destin);
            }

            string[] subdirs = Directory.GetDirectories(srcPath, "*.*", SearchOption.TopDirectoryOnly);
            foreach (string subdir in subdirs)
            {
                string newSubdir = dstPath + subdir.Substring(subdir.LastIndexOf('\\'));
                Directory.CreateDirectory(newSubdir);
                CopyFiles(subdir, newSubdir);
            }
        }

        /// <summary>
        /// Helper method that perform all steps to generate documentation. It's designed to be used outside nant
        /// and is used by console tool.
        /// </summary>
        public void ExecuteDocumenter()
        {
            ExecuteTask();
        }

        /// <summary>
        /// Method that execute all the needed steps to generate documentation from, the passed arguments.
        /// </summary>
        protected override void ExecuteTask()
        {
            string actualFolder = Directory.GetCurrentDirectory();
            try
            {
                // Copy comments to temp folder and construct references.
                if (Directory.Exists(tempFolder))
                    Directory.Delete(tempFolder, true);
                Directory.CreateDirectory(tempFolder);
                Directory.SetCurrentDirectory(tempFolder);

                File.Copy(xmldoc, tempFolder + "\\comments.xml");
                ExecuteCommand(sandcastle + "ProductionTools\\MRefBuilder.EXE", assembly + " /out:reflection.org");
                if (Is11)
                {
                    ExecuteCommand(sandcastle + "ProductionTools\\XslTransform.EXE",
                        " /xsl:\"" + sandcastle + "ProductionTransforms\\ApplyPrototypeDocModel.xsl\"" +
                        " reflection.org" +
                        " /xsl:\"" + sandcastle + "ProductionTransforms\\AddGuidFilenames.xsl\"" +
                        " /out:reflection.xml");
                }
                else
                {
                    ExecuteCommand(sandcastle + "ProductionTools\\XslTransform.EXE",
                        " /xsl:\"" + sandcastle + "ProductionTransforms\\ApplyVSDocModel.xsl\"" +
                        " reflection.org" +
                        " /xsl:\"" + sandcastle + "ProductionTransforms\\AddFriendlyFilenames.xsl\"" +
                        " /out:reflection.xml");
                }

                ExecuteCommand(sandcastle + "ProductionTools\\XslTransform.EXE",
                    " /xsl:\"" + sandcastle + "ProductionTransforms\\ReflectionToManifest.xsl\"" +
                    " reflection.xml" +
                    " /out:manifest.xml");


                // Create folders to content the output in html and copy onto them proper info
                Directory.CreateDirectory("Output");
                Directory.CreateDirectory("Output\\html");
                Directory.CreateDirectory("Output\\icons");
                Directory.CreateDirectory("Output\\scripts");
                Directory.CreateDirectory("Output\\styles");
                Directory.CreateDirectory("Output\\media");
                Directory.CreateDirectory("Intellisense");
                if (Is11)
                {
                    CopyFiles(sandcastle + "presentation\\prototype\\icons", "Output\\icons");
                    CopyFiles(sandcastle + "presentation\\prototype\\scripts", "Output\\scripts");
                    CopyFiles(sandcastle + "presentation\\prototype\\styles", "Output\\styles");
                }
                else
                {
                    CopyFiles(sandcastle + "presentation\\vs2005\\icons", "Output\\icons");
                    CopyFiles(sandcastle + "presentation\\vs2005\\scripts", "Output\\scripts");
                    CopyFiles(sandcastle + "presentation\\vs2005\\styles", "Output\\styles");
                }
                // Construct the manifest
                try
                {
                    ExecuteCommand(sandcastle + "ProductionTools\\BuildAssembler.EXE",
                        "/config:\"" + sandcastle + "presentation\\" + (Is11 ? "prototype" : "vs2005") + "\\Configuration\\sandcastle.config\"" +
                        " manifest.xml");
                }
                catch (BuildException e)
                {
                    if (Is11) throw;
                    throw new BuildException("Error compiling documentation.\n" +
                        "The problem could be than assembly is compiled for .net 1.1 and this doc is generated for .net 2.0\n" +
                        "Set 'IsFor11Framework' property if you're running NANT, or indiates 'net20' as target if using command line\n", e);
                }
                ExecuteCommand(sandcastle + "ProductionTools\\XslTransform.EXE",
                    " /xsl:\"" + sandcastle + "ProductionTransforms\\ReflectionToChmProject.xsl\"" +
                    " reflection.xml" +
                    " /out:Output\\test.hhp");
                if (Is11)
                {
                    ExecuteCommand(sandcastle + "ProductionTools\\XslTransform.EXE",
                        " /xsl:\"" + sandcastle + "ProductionTransforms\\createPrototypetoc.xsl\"" +
                        " reflection.xml" +
                        " /arg:html=Output\\html" +
                        " /out:toc.xml");
                }
                else
                {
                    ExecuteCommand(sandcastle + "ProductionTools\\XslTransform.EXE",
                        " /xsl:\"" + sandcastle + "ProductionTransforms\\createvstoc.xsl\"" +
                        " reflection.xml" +
                        " /arg:html=Output\\html" +
                        " /out:toc.xml");
                }
                ExecuteCommand(sandcastle + "ProductionTools\\XslTransform.EXE",
                    " /xsl:\"" + sandcastle + "ProductionTransforms\\TocToChmContents.xsl\"" +
                    " toc.xml" +
                    " /out:Output\\test.hhc");
                ExecuteCommand(sandcastle + "ProductionTools\\XslTransform.EXE",
                    " /xsl:\"" + sandcastle + "ProductionTransforms\\ReflectionToChmIndex.xsl\"" +
                    " toc.xml" +
                    " /out:Output\\test.hhk");
                try
                {
                    ExecuteCommand(hhc + "\\hhc.exe", "output\\test.hhp");
                }
                catch (BuildException) { }

                // Copy output.
                string target = outFolder + "\\" + Path.GetFileName(Path.ChangeExtension(assembly, ".chm"));
                if (File.Exists("output\\test.chm"))
                {
                    File.Delete(target);
                    File.Copy("output\\test.chm", target);
                }
            }
            finally
            {
                // Return to original folder
                Directory.SetCurrentDirectory(actualFolder);

                // Clean up temporary stuff
                Directory.Delete(tempFolder, true);
            }
        }
    }
}
